;*******************************************************
;ZMR - Zoomin' Mob Routine (B/W Version)
;The Clipped PutSprite by Dux Gregis
;Assembly Coder's Zenith	1999
;*******************************************************

;the following addresses contain the clip lines (must be a multiple of 8):

;for PutSprite
;x@setBottomClip	default is 56
;x@setRightClip		default is 120

;for ClearSprite
;x@setBottomClip0	default is 56
;x@setRightClip0	default is 120

;this is so you can adjust the position of your status bar
;the sprite will clip left and top at 0


;Zoomin' Mob Routine (PutSprite)
;Input: bc = x,y; ix = 8x8 sprite location; hl = background storage buffer
;Output: clipped sprite; all registers destroyed

;the sprite is 8 bytes, followed by an 8 byte mask
;the background storage buffer is 8 bytes


PutSprite:
 ex de,hl							;we'll use DE to hold the storage pointer
 ld a,b
 or a								;test x coordinate
 jp m,clipLeft						;clip left if negative

x@setRightClip	equ		$+1
 sub 120							;if (x coordinate - x clip line >= 0)
 jp p,clipRight						;then clipRight

retClipHorz:						;point of return for both horizontal clips
 ld a,c
 or a								;test the y coordinate
 jp m,clipUp						;if negative, clip top of sprite

x@setBottomClip	equ		$+1
 sub 56								;if (y coordinate - y clip line > 0)
 jp z,ps
 jp p,clipDown						;then clipDown

ps:									;point of return for both vertical clips
 call findPixel						;pixel coordinates -> video address
 ld (x@bitOffset),a					;store findPixel bit offset
x@UpLeft		equ		$
 nop
; ld a,$ff							;dummy instruction

x@clipVert		equ		$+1
 ld b,8								;loop 8 times unless vertical clip
 ld c,0

psLoop:
 push bc							;save loop counter
 push de							;save storage pointer
 push hl							;save video mem

 ld a,(hl)							;background -> HL
 inc hl
 ld l,(hl)
 ld h,a
 ld b,(ix)							;sprite row -> BC
 ld a,(ix+8)						;mask row -> AD
 ld d,c


x@bitOffset		equ		$+1
 ld e,0								;bit offset -> E
 dec e
 jp m,noShift
bitShift:							;shift BC, AD right; HL left E times
 srl b								;sprite
 rr c
 rra								;mask
 rr d
 sla l								;background
 rl h
 dec e
 jp p,bitShift
 inc e								;0 -> E
x@clipHorz		equ		$
 nop								;clear B,A if clip left; C,D if clip right
 nop								;(E is 0, so load into unwanted registers)

noShift:
 ex (sp),hl							;get video mem, push background
 cpl								;invert the sprite mask
 and (hl)							;AND it with video memory
 or b								;OR it with sprite row
 ld (hl),a							;load it to video memory
 inc hl
 ld a,d
 cpl
 and (hl)
 or c
 ld (hl),a

 pop af								;get background
 pop de								;get background storage pointer
 ld (de),a							;store background
 inc de								;and increment pointer

 inc ix								;increment sprite row pointer
 ld bc,15
 add hl,bc							;point to next video row
 pop bc								;loop counter -> B
 djnz psLoop

 ld h,b								;clear HL
 ld l,b
 ld (x@clipHorz),hl					;fix horizontal clipping
 xor a
 ld (x@UpLeft),a
; ld h,$3e
; ld (x@UpLeft),hl					;fix up left clip
 ld a,8
 ld (x@clipVert),a					;fix vertical clipping
 ret

clipUp:
 add a,8							;if (y <= -8)
 ret m								;then return (display nothing)
 ret z
 ld c,0								;new y coordinate is 0
 ld (x@clipVert),a					;new number of sprite rows
 xor %111
 ccf
 adc a,ixl							;add the inverse to IX (sprite row pointer)
 ld ixl,a
 jp nc,ps
 inc ixh
 jp ps

clipDown:
 sub 8								;if (y <= 64)
 ret p								;then return (display nothing)
 neg
 ld (x@clipVert),a					;new number of sprite rows
 jp ps

clipLeft:
 cp -8								;if (x <= -8)
 ret c								;then return (display nothing)
 ret z
 res 7,a							;kill the sign
 ld b,a								;makes for good x coordinate
 dec c								;but for one row down
 ld hl,$437b						;<ld b,e>, <ld a,e> (E is 0)
 ld (x@clipHorz),hl					;move opcodes to clipping area
 jp p,retClipHorz
 inc c
 and 7
 ld b,a
 ld a,$2b							;beastly hack ..
 ld (x@UpLeft),a
 jp retClipHorz

clipRight:
 sub 8
 ret p
 ld hl,$4b53						;<ld c,e>, <ld d,e> (E is 0)
 ld (x@clipHorz),hl					;move opcodes to clipping area
 jp retClipHorz


;Zoomin' Mob Routine (ClearSprite)
;Input: bc = x,y; ix = 8x8 sprite location
;Output: clipped sprite; bc preserved, all others destroyed

;the sprite is 8 bytes, no mask


ClearSprite:
 ld a,b								;clipping is same as PutSprite
 or a
 jp m,clipLeft0

x@setRightClip0		equ		$+1
 sub 120
 jp p,clipRight0

retClipHorz0:
 ld a,c
 or a
 jp m,clipUp0

x@setBottomClip0	equ		$+1
 sub 56
 jp p,clipDown0

cs:
 call findPixel
 ld (x@bitOffset0),a

x@UpLeft0
 nop
; ld a,$ff

x@clipVert0			equ		$+1
 ld b,8
 ld c,0

csLoop:
 push bc
 ld b,(ix)							;sprite
 ld a,$ff							;mask
 ld e,a

x@bitOffset0		equ		$+1
 ld d,0
 dec d
 jp m,noShift0
bitShift0:
 srl b								;sprite
 rr c
 rra								;mask
 dec d
 jp p,bitShift0
 ld e,a
x@clipHorz0			equ		$
 nop
 nop

noShift0:
 cpl
 and (hl)
 or b
 ld (hl),a
 inc hl
 ld a,e
 and (hl)
 or c
 ld (hl),a

 inc ix
 ld bc,15
 add hl,bc
 pop bc
 djnz csLoop

 ld h,b
 ld l,b
 ld (x@clipHorz0),hl
 xor a								;ld a,nn
 ld (x@UpLeft0),a
 ld a,$5f							;ld e,a
 ld (x@clipHorz0-1),a
 ld a,8
 ld (x@clipVert0),a
 ret

clipUp0:
 add a,8
 ret m
 ret z
 ld c,0
 ld (x@clipVert0),a
 jp cs

clipDown0:
 sub 8
 ret p
 neg
 ld (x@clipVert0),a
 jp cs

clipLeft0:
 cp -8
 ret c
 ret z
 res 7,a
 ld b,a
 dec c
 ld hl,$47af						;<ld b,a>, <xor a> (clear A, clear B)
 ld (x@clipHorz0),hl
 jp p,retClipHorz0
 inc c
 and 7
 ld b,a
 ld a,$2b							;<dec hl>
 ld (x@UpLeft0),a
 jp retClipHorz0

clipRight0:
 sub 8
 ret p
 ld hl,$0e5a						;<ld e,d>, <ld c,0> (set E, clear C)
 ld (x@clipHorz0-1),hl				;little endian ;-)
 jp retClipHorz0


findPixel:
 ld a,c
 add a,a
 add a,a
 ld l,a
 ld h,$fc >>2
 ld a,b
 rra
 add hl,hl
 rra
 add hl,hl
 rra
 add a,l
 ld l,a
 ld a,b
 and 7
 ret

.end
